
# q-expansion of newform 2880.2.f.e, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2880
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[2431, [1, 0]], [901, [1, 0]], [641, [1, 0]], [577, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [-2, -1], [0, 2], [2, 0], [0, 6], [0, -2], [0, 0], [0, -4], [0, 0], [-8, 0], [0, -2], [-2, 0], [0, -4], [0, 8], [0, -6], [-10, 0], [-2, 0], [0, 8], [-12, 0], [0, 4], [0, 0], [0, 4], [-10, 0], [0, -8], [-8, 0], [0, 14], [0, 12], [10, 0], [0, 6], [0, 2], [-18, 0], [0, 18], [-20, 0], [20, 0], [-8, 0], [0, -22], [0, 16], [0, -12], [0, 14], [-10, 0], [-2, 0], [-12, 0], [0, 4], [0, 22], [0, 0], [-12, 0], [0, -26], [0, -28], [-10, 0], [0, -14], [20, 0], [22, 0], [-18, 0], [0, 18], [0, -4], [0, 0], [-8, 0], [0, -2], [18, 0], [0, 16], [0, -6], [0, -12], [-12, 0], [0, 4], [0, 2], [8, 0], [0, -28], [0, 12], [10, 0], [0, -14], [0, 0], [0, 2], [0, 6], [20, 0], [0, 16], [-20, 0], [0, -2], [-22, 0], [10, 0], [10, 0], [-22, 0], [-32, 0], [0, 4], [0, 0], [0, -36], [30, 0], [0, 32], [12, 0], [0, -6], [0, 12], [20, 0], [0, -18], [-18, 0], [0, 0], [0, -24], [0, 0], [-22, 0], [0, 16], [38, 0], [0, 28], [0, -18], [0, 44], [-10, 0], [8, 0], [0, 32], [0, 12], [0, 6], [0, 0], [2, 0], [0, 22], [0, 26], [0, -2], [20, 0], [32, 0], [-2, 0], [0, -24], [0, 48], [0, -26], [50, 0], [-2, 0], [0, -36], [0, 2], [0, 4], [8, 0], [32, 0], [30, 0], [-40, 0], [0, -18], [0, -14], [40, 0], [0, -24], [32, 0], [0, -2], [18, 0], [30, 0], [0, 54], [0, -32], [0, 2], [-30, 0], [-52, 0], [-8, 0], [0, -6], [0, -28], [-30, 0], [40, 0], [0, -14], [0, -42], [20, 0], [0, 36], [0, -22], [18, 0], [0, -24], [0, -12], [0, -12], [48, 0], [0, 0], [30, 0], [0, -8], [-28, 0], [0, 12], [0, 46], [0, -38], [-18, 0], [0, -42], [0, 16], [-8, 0], [0, 18], [10, 0], [0, 54], [30, 0], [18, 0], [-32, 0], [0, 44], [-40, 0], [30, 0], [28, 0], [12, 0], [0, 34], [-10, 0], [0, 62], [22, 0], [0, -14], [0, 18], [0, 16], [60, 0], [0, 58], [0, 16], [-50, 0], [-52, 0], [0, -36], [0, 4], [-52, 0], [12, 0], [0, 12], [0, 6], [-18, 0], [0, 6], [0, 18], [0, 16], [-60, 0], [-48, 0], [0, 18], [50, 0], [-50, 0], [0, -18], [0, 0], [0, 4], [-50, 0], [8, 0], [0, 52], [12, 0], [0, 14], [0, 12], [-20, 0], [22, 0], [0, -38], [-22, 0], [0, 8], [0, 34], [18, 0], [-40, 0], [-30, 0], [0, -46], [0, 12], [30, 0], [0, -34], [40, 0], [0, 22], [2, 0], [0, 46], [-20, 0], [32, 0], [18, 0], [0, 56], [0, 28], [-30, 0], [0, -46], [30, 0], [-12, 0], [0, -36], [-32, 0], [0, 34], [10, 0], [0, 26], [60, 0], [0, 42], [42, 0], [-40, 0], [0, -4], [0, -42], [-2, 0], [0, 68], [-70, 0], [0, -46], [-30, 0], [38, 0], [0, 8], [0, 22], [0, -68], [0, 54], [0, -28], [70, 0], [0, -14], [0, 18], [-40, 0], [60, 0], [-2, 0], [0, -64], [0, -66], [-62, 0], [0, 68], [0, -76], [-40, 0], [0, -28], [0, 14], [0, 52], [-70, 0], [-58, 0], [-18, 0], [0, 36], [32, 0], [0, 8], [-22, 0], [0, -52], [28, 0], [0, 24], [0, 82], [40, 0], [50, 0], [-28, 0], [0, 12], [0, -54], [42, 0], [0, 26], [-60, 0], [-48, 0], [0, 54], [-50, 0], [0, -12], [0, -76], [0, 42], [40, 0]]
